@@ -10,6 +10,7 @@
 #include "fitsio2.h"
 
 #define RECBUFLEN 1000
+
 static char stdin_outfile[FLEN_FILENAME];
 
 typedef struct    /* structure containing mem file structure */ 
@@ -29,14 +30,14 @@ typedef struct    /* structure containing mem file structure */
     FILE *fileptr;      /* pointer to compressed output disk file */
 } memdriver;
 
-static memdriver memTable[NIOBUF];  /* allocate mem file handle tables */
+static memdriver memTable[NMAXFILES];  /* allocate mem file handle tables */
 
 /*--------------------------------------------------------------------------*/
 int mem_init(void)
 {
     int ii;
 
-    for (ii = 0; ii < NIOBUF; ii++)  /* initialize all empty slots in table */
+    for (ii = 0; ii < NMAXFILES; ii++) /* initialize all empty slots in table */
     {
        memTable[ii].memaddrptr = 0;
        memTable[ii].memaddr = 0;
@@ -101,27 +102,41 @@ int mem_create_comp(char *filename, int *handle)
 
     /* first, create disk file for the compressed output */
 
-    strcpy(mode, "w+b");    /* open existing file with read-write */
 
-    diskfile = fopen(filename, "r"); /* does file already exist? */
-
-    if (diskfile)
+    if ( !strcmp(filename, "-.gz") || !strcmp(filename, "stdout.gz") ||
+         !strcmp(filename, "STDOUT.gz") )
     {
-        fclose(diskfile);         /* close file and exit with error */
-        return(FILE_NOT_CREATED); 
+       /* special case: create uncompressed FITS file in memory, then
+          compress it an write it out to 'stdout' when it is closed.  */
+
+       diskfile = stdout;
     }
+    else
+    {
+        /* normal case: create disk file for the compressed output */
+
+        strcpy(mode, "w+b");    /* create file with read-write */
+
+        diskfile = fopen(filename, "r"); /* does file already exist? */
+
+        if (diskfile)
+        {
+            fclose(diskfile);         /* close file and exit with error */
+            return(FILE_NOT_CREATED); 
+        }
 
 #if MACHINE == ALPHAVMS || MACHINE == VAXVMS
         /* specify VMS record structure: fixed format, 2880 byte records */
         /* but force stream mode access to enable random I/O access      */
-    diskfile = fopen(filename, mode, "rfm=fix", "mrs=2880", "ctx=stm"); 
+        diskfile = fopen(filename, mode, "rfm=fix", "mrs=2880", "ctx=stm"); 
 #else
-    diskfile = fopen(filename, mode); 
+        diskfile = fopen(filename, mode); 
 #endif
 
-    if (!(diskfile))           /* couldn't create file */
-    {
+        if (!(diskfile))           /* couldn't create file */
+        {
             return(FILE_NOT_CREATED); 
+        }
     }
 
     /* now create temporary memory file */
@@ -152,7 +167,7 @@ int mem_openmem(void **buffptr,   /* I - address of memory pointer          */
     int ii;
 
     *handle = -1;
-    for (ii = 0; ii < NIOBUF; ii++)  /* find empty slot in handle table */
+    for (ii = 0; ii < NMAXFILES; ii++)  /* find empty slot in handle table */
     {
         if (memTable[ii].memaddrptr == 0)
         {
@@ -180,7 +195,7 @@ int mem_createmem(size_t msize, int *handle)
     int ii;
 
     *handle = -1;
-    for (ii = 0; ii < NIOBUF; ii++)  /* find empty slot in handle table */
+    for (ii = 0; ii < NMAXFILES; ii++)  /* find empty slot in handle table */
     {
         if (memTable[ii].memaddrptr == 0)
         {
@@ -235,7 +250,7 @@ int mem_truncate(int handle, OFF_T filesize)
         }
 
         /* if allocated more memory, initialize it to zero */
-        if (filesize > *(memTable[handle].memsizeptr) )
+        if ( (size_t) filesize > *(memTable[handle].memsizeptr) )
         {
              memset(ptr + *(memTable[handle].memsizeptr),
                     0,
@@ -483,7 +498,7 @@ int stdout_close(int handle)
     /* copy from memory to standard out */
     if(fwrite(memTable[handle].memaddr, 1,
               memTable[handle].fitsfilesize, stdout) !=
-              memTable[handle].fitsfilesize )
+              (size_t) memTable[handle].fitsfilesize )
     {
                 ffpmsg("failed to copy memory file to stdout (stdout_close)");
                 status = WRITE_ERROR;
@@ -623,7 +638,8 @@ int mem_compress_open(char *filename, int rwmode, int *hdl)
     }
 
     /* if we allocated too much memory initially, then free it */
-    if (*(memTable[*hdl].memsizeptr) > (memTable[*hdl].fitsfilesize + 256L) ) 
+    if (*(memTable[*hdl].memsizeptr) > 
+       (( (size_t) memTable[*hdl].fitsfilesize) + 256L) ) 
     {
         ptr = realloc(*(memTable[*hdl].memaddrptr), 
                        memTable[*hdl].fitsfilesize);
@@ -687,7 +703,7 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
     short *sptr;
     int status, endian, datatype, bytePerPix, naxis;
     long dim[5] = {1,1,1,1,1}, ii, nvals, offset = 0;
-    size_t filesize = 0;
+    size_t filesize = 0, datasize;
     char rootfile[FLEN_FILENAME], *cptr = 0, *cptr2 = 0;
     void *ptr;
 
@@ -708,7 +724,8 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
         return(URL_PARSE_ERROR);
     }
 
-    strncpy(rootfile, filename, cptr - filename);  /* store the rootname */
+    *rootfile = '\0';
+    strncat(rootfile, filename, cptr - filename);  /* store the rootname */
 
     cptr++;
 
@@ -805,6 +822,7 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
         offset = strtol(cptr+1, 0, 10);
 
     nvals = dim[0] * dim[1] * dim[2] * dim[3] * dim[4];
+    datasize = nvals * bytePerPix;
     filesize = nvals * bytePerPix + 2880;
     filesize = ((filesize - 1) / 2880 + 1) * 2880; 
 
@@ -832,7 +850,7 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
     /* write the required header keywords */
     ffcrim(fptr, datatype, naxis, dim, &status);
 
-    /* close the file, but keep the memory allocated */
+    /* close the FITS file, but keep the memory allocated */
     ffclos(fptr, &status);
 
     if (status > 0)
@@ -846,14 +864,13 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
     if (offset > 0)
        fseek(diskfile, offset, 0);   /* offset to start of the data */
 
-    /* copy the data into memory */
-    cptr = *memTable[*hdl].memaddrptr + 2880;
+    /* read the raw data into memory */
+    ptr = *memTable[*hdl].memaddrptr + 2880;
 
-    /* read bytes */
-    if (fread(cptr, 1, filesize - 2880, diskfile) != filesize - 2880)
+    if (fread((char *) ptr, 1, datasize, diskfile) != datasize)
       status = READ_ERROR;
 
-    fclose(diskfile);
+    fclose(diskfile);  /* close the raw binary disk file */
 
     if (status)
     {
@@ -867,7 +884,7 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
                                  /* efficient way to do this is to just flip  */
                                  /* the most significant bit.                 */
 
-      sptr = (short *) memTable[*hdl].memaddrptr + 2880;
+      sptr = (short *) ptr;
 
       if (endian == BYTESWAPPED)  /* working with native format */
       {
@@ -887,8 +904,6 @@ int mem_rawfile_open(char *filename, int rwmode, int *hdl)
 
     if (endian)  /* swap the bytes if array is in little endian byte order */
     {
-      ptr = *memTable[*hdl].memaddrptr + 2880;
-
       if (datatype == SHORT_IMG || datatype == USHORT_IMG)
       {
         ffswap2( (short *) ptr, nvals);
@@ -937,7 +952,6 @@ int mem_size(int handle, OFF_T *filesize)
 */
 {
     *filesize = memTable[handle].fitsfilesize;
-
     return(0);
 }
 /*--------------------------------------------------------------------------*/
@@ -965,7 +979,8 @@ int mem_close_keep(int handle)
 /*--------------------------------------------------------------------------*/
 int mem_close_comp(int handle)
 /*
-  copy the memory file to stdout, then free the memory
+  compress the memory file, writing it out to the fileptr (which might
+  be stdout)
 */
 {
     int status = 0;
@@ -986,8 +1001,9 @@ int mem_close_comp(int handle)
     memTable[handle].memaddrptr = 0;
     memTable[handle].memaddr = 0;
 
-    /* close the compressed disk file */
-    fclose(memTable[handle].fileptr);
+    /* close the compressed disk file (except if it is 'stdout' */
+    if (memTable[handle].fileptr != stdout)
+        fclose(memTable[handle].fileptr);
 
     return(status);
 }
@@ -1028,7 +1044,8 @@ int mem_write(int hdl, void *buffer, long nbytes)
     size_t newsize;
     char *ptr;
 
-    if (memTable[hdl].currentpos + nbytes > *(memTable[hdl].memsizeptr))
+    if ((size_t) (memTable[hdl].currentpos + nbytes) > 
+         *(memTable[hdl].memsizeptr) )
     {
                
         if (!(memTable[hdl].mem_realloc))
@@ -1044,7 +1061,7 @@ int mem_write(int hdl, void *buffer, long nbytes)
              the defined 'deltasize' parameter
          */
 
-        newsize = maxvalue(
+        newsize = maxvalue( (size_t)
             (((memTable[hdl].currentpos + nbytes - 1) / 2880) + 1) * 2880,
             *(memTable[hdl].memsizeptr) + memTable[hdl].deltasize);
 
