@@ -1,4 +1,4 @@
-/*  Version Info: This file is distributed with version 2.401 of CFITSIO   */
+/*  Version Info: This file is distributed with version 2.500 of CFITSIO   */
 
 /*  The FITSIO software was written by William Pence at the High Energy    */
 /*  Astrophysic Science Archive Research Center (HEASARC) at the NASA      */
@@ -37,6 +37,11 @@ SERVICES PROVIDED HEREUNDER."
 #define _FITSIO_H
 
 #include <stdio.h>
+
+#if defined(linux) || defined(__APPLE__)
+#  include <sys/types.h>  /* apparently needed on debian linux systems */
+#endif                    /* to define off_t                           */
+
 #include <stdlib.h>  /* apparently needed to define size_t with gcc 2.8.1 */
 #include <limits.h>  /* needed for LLONG_MAX and INT64_MAX definitions */
 
@@ -47,7 +52,9 @@ SERVICES PROVIDED HEREUNDER."
 /* on whether _LARGEFILE_SOURCE is defined in sys/feature_tests.h  */
 /* (at least on Solaris platforms using cc)  */
 
-#if defined(_OFF_T) || defined(_MIPS_SZLONG) || defined(__APPLE__)
+/*  Debian systems require the 2nd test, below,         */
+/*  i.e, "(defined(linux) && defined(__off_t_defined))" */
+#if defined(_OFF_T) || (defined(linux) && defined(__off_t_defined)) || defined(_MIPS_SZLONG) || defined(__APPLE__) || defined(_AIX)
 #    define OFF_T off_t
 #else
 #    define OFF_T long
@@ -60,7 +67,7 @@ SERVICES PROVIDED HEREUNDER."
 #   ifndef HAVE_LONGLONG
 #      define HAVE_LONGLONG 1
 #   endif
-#elif defined(_MSC_VER) || defined(__BORLANDC__)  /* Windows PCs */
+#elif defined(_MSC_VER)   /* Windows PCs; Visual C++, but not Borland C++ */
     typedef __int64 LONGLONG;
 #   ifndef HAVE_LONGLONG
 #      define HAVE_LONGLONG 1
@@ -86,6 +93,7 @@ SERVICES PROVIDED HEREUNDER."
  
 #define TBIT          1  /* codes for FITS table data types */
 #define TBYTE        11
+#define TSBYTE       12
 #define TLOGICAL     14
 #define TSTRING      16
 #define TUSHORT      20
@@ -134,6 +142,7 @@ SERVICES PROVIDED HEREUNDER."
                          /* datatypes; these codes are only used internally */
                          /* within cfitsio to make it easier for users      */
                          /* to deal with unsigned integers.                 */
+#define SBYTE_IMG    10
 #define USHORT_IMG   20
 #define ULONG_IMG    40
 
@@ -168,8 +177,6 @@ SERVICES PROVIDED HEREUNDER."
 #define CASESEN   1   /* do case-sensitive string match */
 #define CASEINSEN 0   /* do case-insensitive string match */
  
-#define MAXHDU 1000    /* maximum number of extensions allowed in a FITS file */
-
 #define GT_ID_ALL_URI  0   /* hierarchical grouping parameters */
 #define GT_ID_REF      1
 #define GT_ID_POS      2
@@ -230,7 +237,8 @@ typedef struct      /* structure used to store basic FITS file information */
     int hdutype;    /* 0 = primary array, 1 = ASCII table, 2 = binary table */
     int writemode;  /* 0 = readonly, 1 = readwrite */
     int maxhdu;     /* highest numbered HDU known to exist in the file */
-    OFF_T headstart[MAXHDU + 1]; /* byte offset in file to start of each HDU */
+    int MAXHDU;     /* dynamically allocated dimension of headstart array */
+    OFF_T *headstart; /* byte offset in file to start of each HDU */
     OFF_T headend;  /* byte offest in file to end of the current HDU header */
     OFF_T nextkey;  /* byte offset in file to beginning of next keyword */
     OFF_T datastart;/* byte offset in file to start of the current data unit */
@@ -265,6 +273,8 @@ typedef struct      /* structure used to store basic FITS file information */
 
     double zscale;          /* scaling value, if same for all tiles */
     double zzero;           /* zero pt, if same for all tiles */
+    double cn_bscale;       /* value of the BSCALE keyword in header */
+    double cn_bzero;        /* value of the BZERO keyword in header */
     int zblank;             /* value for null pixels, if not a column */
 
     int rice_blocksize;     /* first compression parameter */
@@ -303,7 +313,10 @@ typedef struct  /* structure for the iterator function column information */
 #define OutputCol        2  /* flag for output only iterator column      */
 
 /* error status codes */
- 
+
+#define SKIP_TABLE       -104 /* move to 1st image when opening file */
+#define SKIP_IMAGE       -103 /* move to 1st table when opening file */
+#define SKIP_NULL_PRIMARY -102 /* skip null primary array when opening file */
 #define USE_MEM_BUFF     -101  /* use memory buffer when opening file */
 #define OVERFLOW_ERR      -11  /* overflow during datatype conversion */
 #define PREPEND_PRIMARY    -9  /* used in ffiimg to insert new primary array */
@@ -327,6 +340,7 @@ typedef struct  /* structure for the iterator function column information */
 #define DRIVER_INIT_FAILED 123  /* driver initialization failed */
 #define NO_MATCHING_DRIVER 124  /* matching driver is not registered */
 #define URL_PARSE_ERROR    125  /* failed to parse input file URL */
+#define RANGE_PARSE_ERROR  126  /* failed to parse input file URL */
 
 #define	SHARED_ERRBASE	(150)
 #define	SHARED_BADARG	(SHARED_ERRBASE + 1)
@@ -475,10 +489,12 @@ extern "C" {
 
 /*----------------  FITS file URL parsing routines -------------*/
 int fits_get_token(char **ptr, char *delimiter, char *token, int *isanumber);
+char *fits_split_names(char *list);
 int ffiurl(char *url,  char *urltype, char *infile,
                     char *outfile, char *extspec, char *rowfilter,
                     char *binspec, char *colspec, int *status);
 int ffrtnm(char *url, char *rootname, int *status);
+int ffexist(const char *infile, int *exists, int *status);
 int ffexts(char *extspec, int *extnum,  char *extname, int *extvers,
           int *hdutype, char *colname, char *rowexpress, int *status);
 int ffextn(char *url, int *extension_num, int *status);
@@ -493,6 +509,8 @@ int ffbinr(char **binspec, char *colname, double *minin,
                         double *maxin, double *binsizein, char *minname,
                         char *maxname, char *binname, int *status);
 int ffimport_file( char *filename, char **contents, int *status );
+int ffrwrg( char *rowlist, long maxrows, int maxranges, int *numranges,
+      long *minrow, long *maxrow, int *status);
 
 /*----------------  FITS file I/O routines -------------*/
 int ffomem(fitsfile **fptr, const char *name, int mode, void **buffptr,
@@ -500,6 +518,9 @@ int ffomem(fitsfile **fptr, const char *name, int mode, void **buffptr,
            void *(*mem_realloc)(void *p, size_t newsize),
            int *status);
 int ffopen(fitsfile **fptr, const char *filename, int iomode, int *status);
+int ffdopn(fitsfile **fptr, const char *filename, int iomode, int *status);
+int fftopn(fitsfile **fptr, const char *filename, int iomode, int *status);
+int ffiopn(fitsfile **fptr, const char *filename, int iomode, int *status);
 int ffreopen(fitsfile *openfptr, fitsfile **newfptr, int *status); 
 int ffinit(fitsfile **fptr, const char *filename, int *status);
 int ffimem(fitsfile **fptr,  void **buffptr,
@@ -509,6 +530,7 @@ int ffimem(fitsfile **fptr,  void **buffptr,
 int fftplt(fitsfile **fptr, const char *filename, const char *tempname,
            int *status);
 int ffflus(fitsfile *fptr, int *status);
+int ffflsh(fitsfile *fptr, int clearbuf, int *status);
 int ffclos(fitsfile *fptr, int *status);
 int ffdelt(fitsfile *fptr, int *status);
 int ffflnm(fitsfile *fptr, char *filename, int *status);
@@ -519,8 +541,10 @@ float ffvers(float *version);
 void ffupch(char *string);
 void ffgerr(int status, char *errtext);
 void ffpmsg(const char *err_message);
+void ffpmrk(void);
 int  ffgmsg(char *err_message);
 void ffcmsg(void);
+void ffcmrk(void);
 void ffrprt(FILE *stream, int status);
 void ffcmps(char *templt, char *colname, int  casesen, int *match,
            int *exact);
@@ -538,6 +562,8 @@ int ffasfm(char *tform, int *datacode, long *width, int *decim, int *status);
 int ffbnfm(char *tform, int *datacode, long *repeat, long *width, int *status);
 int ffgabc(int tfields, char **tform, int space, long *rowlen, long *tbcol,
            int *status);
+int fits_get_section_range(char **ptr,long *secmin,long *secmax,long *incre,
+              int *status);
  
 /*----------------- write single keywords --------------*/
 int ffpky(fitsfile *fptr, int datatype, char *keyname, void *value,
@@ -660,7 +686,9 @@ int ffgkne(fitsfile *fptr, char *keyname, int nstart, int nmax, float *value,
 int ffgknd(fitsfile *fptr, char *keyname, int nstart, int nmax, double *value,
            int *nfound, int *status);
 int ffh2st(fitsfile *fptr, char **header, int  *status);
- 
+int ffhdr2str( fitsfile *fptr,  int exclude_comm, char **exclist,
+   int nexc, char **header, int *nkeys, int  *status);
+
 /*----------------- read required header keywords --------------*/
 int ffghpr(fitsfile *fptr, int maxdim, int *simple, int *bitpix, int *naxis,
           long naxes[], long *pcount, long *gcount, int *extend, int *status);
@@ -728,6 +756,7 @@ int ffmkfm(fitsfile *fptr, char *keyname, double *value, int decim, char *comm,
  
 /*--------------------- insert keywords ---------------*/
 int ffirec(fitsfile *fptr, int nkey, char *card, int *status);
+int ffikey(fitsfile *fptr, char *card, int *status);
 int ffikyu(fitsfile *fptr, char *keyname, char *comm, int *status);
 int ffikys(fitsfile *fptr, char *keyname, char *value, char *comm,int *status);
 int ffikls(fitsfile *fptr, char *keyname, char *value, char *comm,int *status);
@@ -764,6 +793,7 @@ int ffghof(fitsfile *fptr, OFF_T *headstart, OFF_T *datastart, OFF_T *dataend,
 int ffgipr(fitsfile *fptr, int maxaxis, int *imgtype, int *naxis,
            long *naxes, int *status);
 int ffgidt(fitsfile *fptr, int *imgtype, int *status);
+int ffgiet(fitsfile *fptr, int *imgtype, int *status);
 int ffgidm(fitsfile *fptr, int *naxis,  int *status);
 int ffgisz(fitsfile *fptr, int nlen, long *naxes, int *status);
 
@@ -785,6 +815,8 @@ int ffibin(fitsfile *fptr,long naxis2, int tfields, char **ttype, char **tform,
 int ffrsim(fitsfile *fptr, int bitpix, int naxis, long *naxes, int *status);
 int ffdhdu(fitsfile *fptr, int *hdutype, int *status);
 int ffcopy(fitsfile *infptr, fitsfile *outfptr, int morekeys, int *status);
+int ffcpfl(fitsfile *infptr, fitsfile *outfptr, int prev, int cur, int follow,
+            int *status);
 int ffcphd(fitsfile *infptr, fitsfile *outfptr, int *status);
 int ffcpdt(fitsfile *infptr, fitsfile *outfptr, int *status);
 int ffchfl(fitsfile *fptr, int *status);
@@ -818,6 +850,8 @@ int ffgcnn(fitsfile *fptr, int casesen, char *templt, char *colname,
  
 int ffgtcl(fitsfile *fptr, int colnum, int *typecode, long *repeat,
            long *width, int *status);
+int ffeqty(fitsfile *fptr, int colnum, int *typecode, long *repeat,
+           long *width, int *status);
 int ffgncl(fitsfile *fptr, int  *ncols, int *status);
 int ffgnrw(fitsfile *fptr, long *nrows, int *status);
 int ffgacl(fitsfile *fptr, int colnum, char *ttype, long *tbcol,
@@ -842,6 +876,8 @@ int ffgpf(fitsfile *fptr, int  datatype, long firstelem, long nelem,
           void *array, char *nullarray, int  *anynul, int  *status);
 int ffgpvb(fitsfile *fptr, long group, long firstelem, long nelem, unsigned
            char nulval, unsigned char *array, int *anynul, int *status);
+int ffgpvsb(fitsfile *fptr, long group, long firstelem, long nelem, signed
+           char nulval, signed char *array, int *anynul, int *status);
 int ffgpvui(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned short nulval, unsigned short *array, int *anynul, 
            int *status);
@@ -865,6 +901,8 @@ int ffgpvd(fitsfile *fptr, long group, long firstelem, long nelem,
  
 int ffgpfb(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned char *array, char *nularray, int *anynul, int *status);
+int ffgpfsb(fitsfile *fptr, long group, long firstelem, long nelem,
+           signed char *array, char *nularray, int *anynul, int *status);
 int ffgpfui(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned short *array, char *nularray, int *anynul, int *status);
 int ffgpfi(fitsfile *fptr, long group, long firstelem, long nelem,
@@ -887,6 +925,9 @@ int ffgpfd(fitsfile *fptr, long group, long firstelem, long nelem,
 int ffg2db(fitsfile *fptr, long group, unsigned char nulval, long ncols,
            long naxis1, long naxis2, unsigned char *array,
            int *anynul, int *status);
+int ffg2dsb(fitsfile *fptr, long group, signed char nulval, long ncols,
+           long naxis1, long naxis2, signed char *array,
+           int *anynul, int *status);
 int ffg2dui(fitsfile *fptr, long group, unsigned short nulval, long ncols,
            long naxis1, long naxis2, unsigned short *array,
            int *anynul, int *status);
@@ -918,6 +959,9 @@ int ffg2dd(fitsfile *fptr, long group, double nulval, long ncols,
 int ffg3db(fitsfile *fptr, long group, unsigned char nulval, long ncols,
            long nrows, long naxis1, long naxis2, long naxis3,
            unsigned char *array, int *anynul, int *status);
+int ffg3dsb(fitsfile *fptr, long group, signed char nulval, long ncols,
+           long nrows, long naxis1, long naxis2, long naxis3,
+           signed char *array, int *anynul, int *status);
 int ffg3dui(fitsfile *fptr, long group, unsigned short nulval, long ncols,
            long nrows, long naxis1, long naxis2, long naxis3,
            unsigned short *array, int *anynul, int *status);
@@ -949,6 +993,9 @@ int ffg3dd(fitsfile *fptr, long group, double nulval, long ncols,
 int ffgsvb(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
   long *trc, long *inc, unsigned char nulval, unsigned char *array,
   int *anynul, int *status);
+int ffgsvsb(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
+  long *trc, long *inc, signed char nulval, signed char *array,
+  int *anynul, int *status);
 int ffgsvui(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
   long *trc, long *inc, unsigned short nulval, unsigned short *array, 
   int *anynul, int *status);
@@ -976,6 +1023,9 @@ int ffgsvd(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
 int ffgsfb(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
   long *trc, long *inc, unsigned char *array, char *flagval,
   int *anynul, int *status);
+int ffgsfsb(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
+  long *trc, long *inc, signed char *array, char *flagval,
+  int *anynul, int *status);
 int ffgsfui(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
   long *trc, long *inc, unsigned short *array, char *flagval, int *anynul, 
   int *status);
@@ -1002,6 +1052,8 @@ int ffgsfd(fitsfile *fptr, int colnum, int naxis, long *naxes, long *blc,
  
 int ffggpb(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned char *array, int *status);
+int ffggpsb(fitsfile *fptr, long group, long firstelem, long nelem,
+           signed char *array, int *status);
 int ffggpui(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned short *array, int *status);
 int ffggpi(fitsfile *fptr, long group, long firstelem, long nelem,
@@ -1037,6 +1089,9 @@ int ffgcvl (fitsfile *fptr, int colnum, long firstrow, long firstelem,
 int ffgcvb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned char nulval, unsigned char *array,
            int *anynul, int *status);
+int ffgcvsb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
+           long nelem, signed char nulval, signed char *array,
+           int *anynul, int *status);
 int ffgcvui(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned short nulval, unsigned short *array, 
            int *anynul, int *status);
@@ -1076,6 +1131,8 @@ int ffgcfl(fitsfile *fptr, int colnum, long firstrow, long firstelem, long
           nelem, char *array, char *nularray, int *anynul, int *status);
 int ffgcfb(fitsfile *fptr, int colnum, long firstrow, long firstelem, long
       nelem, unsigned char *array, char *nularray, int *anynul, int *status);
+int ffgcfsb(fitsfile *fptr, int colnum, long firstrow, long firstelem, long
+      nelem, signed char *array, char *nularray, int *anynul, int *status);
 int ffgcfui(fitsfile *fptr, int colnum, long firstrow, long firstelem,
       long nelem, unsigned short *array, char *nularray, int *anynul, 
       int *status);
@@ -1124,6 +1181,8 @@ int ffppr(fitsfile *fptr, int datatype, long  firstelem, long nelem,
           void *array, int *status);
 int ffpprb(fitsfile *fptr, long group, long firstelem,
            long nelem, unsigned char *array, int *status);
+int ffpprsb(fitsfile *fptr, long group, long firstelem,
+           long nelem, signed char *array, int *status);
 int ffpprui(fitsfile *fptr, long group, long firstelem,
            long nelem, unsigned short *array, int *status);
 int ffppri(fitsfile *fptr, long group, long firstelem,
@@ -1151,6 +1210,8 @@ int ffppn(fitsfile *fptr, int datatype, long  firstelem, long  nelem,
           void  *array, void *nulval, int  *status);
 int ffppnb(fitsfile *fptr, long group, long firstelem, long nelem,
            unsigned char *array, unsigned char nulval, int *status);
+int ffppnsb(fitsfile *fptr, long group, long firstelem, long nelem,
+           signed char *array, signed char nulval, int *status);
 int ffppnui(fitsfile *fptr, long group, long firstelem,
            long nelem, unsigned short *array, unsigned short nulval,
            int *status);
@@ -1173,6 +1234,8 @@ int ffppnjj(fitsfile *fptr, long group, long firstelem,
 
 int ffp2db(fitsfile *fptr, long group, long ncols, long naxis1,
            long naxis2, unsigned char *array, int *status);
+int ffp2dsb(fitsfile *fptr, long group, long ncols, long naxis1,
+           long naxis2, signed char *array, int *status);
 int ffp2dui(fitsfile *fptr, long group, long ncols, long naxis1,
            long naxis2, unsigned short *array, int *status);
 int ffp2di(fitsfile *fptr, long group, long ncols, long naxis1,
@@ -1194,6 +1257,8 @@ int ffp2djj(fitsfile *fptr, long group, long ncols, long naxis1,
 
 int ffp3db(fitsfile *fptr, long group, long ncols, long nrows, long naxis1,
            long naxis2, long naxis3, unsigned char *array, int *status);
+int ffp3dsb(fitsfile *fptr, long group, long ncols, long nrows, long naxis1,
+           long naxis2, long naxis3, signed char *array, int *status);
 int ffp3dui(fitsfile *fptr, long group, long ncols, long nrows, long naxis1,
            long naxis2, long naxis3, unsigned short *array, int *status);
 int ffp3di(fitsfile *fptr, long group, long ncols, long nrows, long naxis1,
@@ -1217,6 +1282,8 @@ int ffpss(fitsfile *fptr, int datatype,
            long *fpixel, long *lpixel, void *array, int *status);
 int ffpssb(fitsfile *fptr, long group, long naxis, long *naxes,
            long *fpixel, long *lpixel, unsigned char *array, int *status);
+int ffpsssb(fitsfile *fptr, long group, long naxis, long *naxes,
+           long *fpixel, long *lpixel, signed char *array, int *status);
 int ffpssui(fitsfile *fptr, long group, long naxis, long *naxes,
            long *fpixel, long *lpixel, unsigned short *array, int *status);
 int ffpssi(fitsfile *fptr, long group, long naxis, long *naxes,
@@ -1238,6 +1305,8 @@ int ffpssjj(fitsfile *fptr, long group, long naxis, long *naxes,
 
 int ffpgpb(fitsfile *fptr, long group, long firstelem,
            long nelem, unsigned char *array, int *status);
+int ffpgpsb(fitsfile *fptr, long group, long firstelem,
+           long nelem, signed char *array, int *status);
 int ffpgpui(fitsfile *fptr, long group, long firstelem,
            long nelem, unsigned short *array, int *status);
 int ffpgpi(fitsfile *fptr, long group, long firstelem,
@@ -1294,6 +1363,8 @@ int ffpcll(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, char *array, int *status);
 int ffpclb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned char *array, int *status);
+int ffpclsb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
+           long nelem, signed char *array, int *status);
 int ffpclui(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned short *array, int *status);
 int ffpcli(fitsfile *fptr, int colnum, long firstrow, long firstelem,
@@ -1330,6 +1401,9 @@ int ffpcnl( fitsfile *fptr, int  colnum, long  firstrow, long  firstelem,
 int ffpcnb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned char *array, unsigned char nulvalue,
            int *status);
+int ffpcnsb(fitsfile *fptr, int colnum, long firstrow, long firstelem,
+           long nelem, signed char *array, signed char nulvalue,
+           int *status);
 int ffpcnui(fitsfile *fptr, int colnum, long firstrow, long firstelem,
            long nelem, unsigned short *array, unsigned short nulvalue,
            int *status);
@@ -1360,6 +1434,7 @@ int ffptbb(fitsfile *fptr, long firstrow, long firstchar, long nchars,
  
 int ffirow(fitsfile *fptr, long firstrow, long nrows, int *status);
 int ffdrow(fitsfile *fptr, long firstrow, long nrows, int *status);
+int ffdrrg(fitsfile *fptr, char *ranges, int *status);
 int ffdrws(fitsfile *fptr, long *rownum,  long nrows, int *status);
 int fficol(fitsfile *fptr, int numcol, char *ttype, char *tform, int *status);
 int fficls(fitsfile *fptr, int firstcol, int ncols, char **ttype,
