@@ -3,14 +3,36 @@
  
 #include "fitsio.h"
 
-#define SUPPORT_64BIT_INTEGERS 0
+/* Setting SUPPORT_64_BIT_INTEGERS to 1 will enable CFITSIO to read */
+/* and write images with BITPIX = 64 and binary table columns with  */
+/* TFORMn = 'K'.  Otherwise, setting SUPPORT_64_BIT_INTEGERS to 0   */
+/* will cause CFITSIO to not recognize these  non-standard 64-bit   */
+/* FITS datatypes.                                                  */
+
+#define SUPPORT_64BIT_INTEGERS 1
+
+/*
+  If REPLACE_LINKS is defined, then whenever CFITSIO fails to open
+  a file with write access because it is a soft link to a file that
+  only has read access, then CFITSIO will attempt to replace
+  the link with a local copy of the file, with write access.  This
+  feature was originally added to support the ftools in the Hera
+  environment, where many of the user's data file are soft links.
+*/
+#if defined(BUILD_HERA)
+#define REPLACE_LINKS 1
+#endif
 
 #define USE_LARGE_VALUE -99  /* flag used when writing images */
 
 #define DBUFFSIZE 28800 /* size of data buffer in bytes */
 
-#define NIOBUF  25       /* number of IO buffers to create */
-#define IOBUFLEN 2880    /* size in bytes of each IO buffer */
+#define NIOBUF  40  /* number of IO buffers to create */
+         /* !! Significantly increasing NIOBUF may degrade performance !! */
+#define NMAXFILES  300   /* maximum number of FITS files that can be opened */
+        /* CFITSIO will allocate (NMAXFILES * 80) bytes of memory */
+
+#define IOBUFLEN 2880    /* size in bytes of each IO buffer (DONT CHANGE!) */
 #define MINDIRECT 8640   /* minimum size for direct reads and writes */
                          /* MINDIRECT must have a value >= 8640 */
 
@@ -21,6 +43,7 @@
 #define ALPHAVMS           4
 #define IBMPC              5
 #define CRAY               6
+#define PC64BIT            7
 
 #define GFLOAT             1
 #define IEEEFLOAT          2
@@ -77,6 +100,13 @@
 #define MACHINE ULTRIX
 #define BYTESWAPPED TRUE
  
+#elif defined(__sparcv9)
+
+/*  SUN Solaris7 in 64-bit mode */
+#define BYTESWAPPED FALSE
+#define MACHINE NATIVE
+#define LONGSIZE 64   
+
 #elif defined(__i386) || defined(__i386__) || defined(__i486__) || defined(__i586__) 
 
 /*  IBM PC */
@@ -89,23 +119,17 @@
 #define MACHINE IBMPC       
 #define BYTESWAPPED TRUE
 
-#elif defined(_NI_mswin_)
+#elif defined(_NI_mswin_) || defined(__EMX__)
 
-/*  LabWindows/CVI with Windows 3.x, 95, or NT  */
+/*  LabWindows/CVI with Windows, or PC runnin OS/2  */
 #define MACHINE IBMPC       
 #define BYTESWAPPED TRUE
 
-#elif  defined(__EMX__)
+#elif defined(__ia64__)  || defined(__x86_64__)
 
-/*  IBM PC running OS/2 */
-#define MACHINE IBMPC
+/*  Intel itanium 64-bit PC, or AMD opteron 64-bit PC */
 #define BYTESWAPPED TRUE
-
-#elif defined(__sparcv9)
-
-/*  SUN Solaris7 in 64-bit mode */
-#define BYTESWAPPED FALSE
-#define MACHINE NATIVE
+#define MACHINE PC64BIT
 #define LONGSIZE 64   
 
 #else
@@ -176,6 +200,8 @@
  
 #endif
 
+#define DSCHAR_MAX  127.49 /* max double value that fits in an signed char */
+#define DSCHAR_MIN -128.49 /* min double value that fits in an signed char */
 #define DUCHAR_MAX  255.49 /* max double value that fits in an unsigned char */
 #define DUCHAR_MIN -0.49   /* min double value that fits in an unsigned char */
 #define DUSHRT_MAX  65535.49 /* max double value that fits in a unsigned short*/
@@ -217,6 +243,10 @@
 #define LONGLONG_MAX LLONG_MAX
 #define LONGLONG_MIN LLONG_MIN
 
+#elif defined(LONG_LONG_MAX)
+#define LONGLONG_MAX LONG_LONG_MAX
+#define LONGLONG_MIN LONG_LONG_MIN
+
 #elif defined(INT64_MAX)
 #define LONGLONG_MAX INT64_MAX
 #define LONGLONG_MIN INT64_MIN
@@ -271,7 +301,6 @@ int ffgttb(fitsfile *fptr, long *rowlen, long *nrows, long *pcount,
           long *tfield, int *status);
  
 int ffmkey(fitsfile *fptr, char *card, int *status);
-int ffikey(fitsfile *fptr, char *card, int *status);
  
 int ffmbyt(fitsfile *fptr, OFF_T bytpos, int ignore_err, int *status);
 int ffgbyt(fitsfile *fptr, long nbytes, void *buffer, int *status);
@@ -282,8 +311,6 @@ int ffpbytoff(fitsfile *fptr, long gsize, long ngroups, long offset,
            void *buffer, int *status);
 int ffldrc(fitsfile *fptr, long record, int err_mode, int *status);
 int ffwhbf(fitsfile *fptr, int *nbuff);
-int ffcurbuf(int nbuff, FITSfile **Fptr);
-int ffflsh(fitsfile *fptr, int clearbuf, int *status);
 int ffbfeof(fitsfile *fptr, int *status);
 int ffbfwt(int nbuff, int *status);
 int fits_get_num_files(void);
@@ -294,6 +321,8 @@ int ffourl(char *url, char *urltype, char *outfile, char *tmplfile,
 int ffparsecompspec(fitsfile *fptr, char *compspec, int *status);
 int ffoptplt(fitsfile *fptr, const char *tempname, int *status);
 int fits_is_this_a_copy(char *urltype);
+int fits_store_Fptr(FITSfile *Fptr, int *status);
+int fits_clear_Fptr(FITSfile *Fptr, int *status);
 int fits_already_open(fitsfile **fptr, char *url, 
     char *urltype, char *infile, char *extspec, char *rowfilter,
     char *binspec, char *colspec, int  mode,int  *isopen, int  *status);
@@ -307,8 +336,6 @@ int ffcalchist(long totalrows, long offset, long firstrow, long nrows,
 int fits_copy_image_cell(fitsfile **fptr, char *outfile, char *colname,
            long rownum, int *status);
 int fits_copy_image_keywords(fitsfile *infptr, fitsfile *outfptr, int *status);
-int fits_get_section_range(char **ptr,long *secmin,long *secmax,long *incre,
-              int *status);
 int ffrhdu(fitsfile *fptr, int *hdutype, int *status);
 int ffpinit(fitsfile *fptr, int *status);
 int ffainit(fitsfile *fptr, int *status);
@@ -340,6 +367,9 @@ int ffwrite(FITSfile *fptr, long nbytes, void *buffer,
             int *status);
 int fftrun(fitsfile *fptr, OFF_T filesize, int *status);
 
+int ffpcluc(fitsfile *fptr, int colnum, long firstrow, long firstelem, long
+           nelem, int *status);
+	   
 int ffgcll(fitsfile *fptr, int colnum, long firstrow, OFF_T firstelem, long
            nelem, int nultyp, char nulval, char *array, char *nularray,
            int *anynul, int *status);
@@ -352,6 +382,9 @@ int ffgcls2(fitsfile *fptr, int colnum, long firstrow, long firstelem,
 int ffgclb(fitsfile *fptr, int colnum, long firstrow, OFF_T firstelem,
            long nelem, long  elemincre, int nultyp, unsigned char nulval,
            unsigned char *array, char *nularray, int *anynul, int  *status);
+int ffgclsb(fitsfile *fptr, int colnum, long firstrow, OFF_T firstelem,
+           long nelem, long  elemincre, int nultyp, signed char nulval,
+           signed char *array, char *nularray, int *anynul, int  *status);
 int ffgclui(fitsfile *fptr, int colnum, long firstrow, OFF_T firstelem,
            long nelem, long  elemincre, int nultyp, unsigned short nulval,
            unsigned short *array, char *nularray, int *anynul, int  *status);
@@ -431,6 +464,29 @@ int fffstri1(char *input, long ntodo, double scale, double zero,
             unsigned char nullval, char *nullarray, int *anynull,
             unsigned char *output, int *status);
  
+int fffi1s1(unsigned char *input, long ntodo, double scale, double zero,
+            int nullcheck, unsigned char tnull, signed char nullval, char
+             *nullarray, int *anynull, signed char *output, int *status);
+int fffi2s1(short *input, long ntodo, double scale, double zero,
+            int nullcheck, short tnull, signed char nullval, char *nullarray,
+            int *anynull, signed char *output, int *status);
+int fffi4s1(INT32BIT *input, long ntodo, double scale, double zero,
+            int nullcheck, INT32BIT tnull, signed char nullval, char *nullarray,
+            int *anynull, signed char *output, int *status);
+int fffi8s1(LONGLONG *input, long ntodo, double scale, double zero,
+            int nullcheck, long tnull, signed char nullval, char *nullarray,
+            int *anynull, signed char *output, int *status);
+int fffr4s1(float *input, long ntodo, double scale, double zero,
+            int nullcheck, signed char nullval, char *nullarray,
+            int *anynull, signed char *output, int *status);
+int fffr8s1(double *input, long ntodo, double scale, double zero,
+            int nullcheck, signed char nullval, char *nullarray,
+            int *anynull, signed char *output, int *status);
+int fffstrs1(char *input, long ntodo, double scale, double zero,
+            long twidth, double power, int nullcheck, char *snull,
+            signed char nullval, char *nullarray, int *anynull,
+            signed char *output, int *status);
+
 int fffi1u2(unsigned char *input, long ntodo, double scale, double zero,
             int nullcheck, unsigned char tnull, unsigned short nullval, 
             char *nullarray,
@@ -642,6 +698,8 @@ int fffstrr8(char *input, long ntodo, double scale, double zero,
  
 int ffi1fi1(unsigned char *array, long ntodo, double scale, double zero,
             unsigned char *buffer, int *status);
+int ffs1fi1(signed char *array, long ntodo, double scale, double zero,
+            unsigned char *buffer, int *status);
 int ffu2fi1(unsigned short *array, long ntodo, double scale, double zero,
             unsigned char *buffer, int *status);
 int ffi2fi1(short *array, long ntodo, double scale, double zero,
@@ -663,6 +721,8 @@ int ffr8fi1(double *array, long ntodo, double scale, double zero,
  
 int ffi1fi2(unsigned char *array, long ntodo, double scale, double zero,
             short *buffer, int *status);
+int ffs1fi2(signed char *array, long ntodo, double scale, double zero,
+            short *buffer, int *status);
 int ffu2fi2(unsigned short *array, long ntodo, double scale, double zero,
             short *buffer, int *status);
 int ffi2fi2(short *array, long ntodo, double scale, double zero,
@@ -684,6 +744,8 @@ int ffr8fi2(double *array, long ntodo, double scale, double zero,
  
 int ffi1fi4(unsigned char *array, long ntodo, double scale, double zero,
             INT32BIT *buffer, int *status);
+int ffs1fi4(signed char *array, long ntodo, double scale, double zero,
+            INT32BIT *buffer, int *status);
 int ffu2fi4(unsigned short *array, long ntodo, double scale, double zero,
             INT32BIT *buffer, int *status);
 int ffi2fi4(short *array, long ntodo, double scale, double zero,
@@ -711,6 +773,8 @@ int ffi2fi8(short *array, long ntodo, double scale, double zero,
             LONGLONG *buffer, int *status);
 int ffi1fi8(unsigned char *array, long ntodo, double scale, double zero,
             LONGLONG *buffer, int *status);
+int ffs1fi8(signed char *array, long ntodo, double scale, double zero,
+            LONGLONG *buffer, int *status);
 int ffr4fi8(float *array, long ntodo, double scale, double zero,
             LONGLONG *buffer, int *status);
 int ffr8fi8(double *array, long ntodo, double scale, double zero,
@@ -726,6 +790,8 @@ int ffuintfi8(unsigned int *array, long ntodo, double scale, double zero,
 
 int ffi1fr4(unsigned char *array, long ntodo, double scale, double zero,
             float *buffer, int *status);
+int ffs1fr4(signed char *array, long ntodo, double scale, double zero,
+            float *buffer, int *status);
 int ffu2fr4(unsigned short *array, long ntodo, double scale, double zero,
             float *buffer, int *status);
 int ffi2fr4(short *array, long ntodo, double scale, double zero,
@@ -747,6 +813,8 @@ int ffr8fr4(double *array, long ntodo, double scale, double zero,
  
 int ffi1fr8(unsigned char *array, long ntodo, double scale, double zero,
             double *buffer, int *status);
+int ffs1fr8(signed char *array, long ntodo, double scale, double zero,
+            double *buffer, int *status);
 int ffu2fr8(unsigned short *array, long ntodo, double scale, double zero,
             double *buffer, int *status);
 int ffi2fr8(short *array, long ntodo, double scale, double zero,
@@ -768,6 +836,8 @@ int ffr8fr8(double *array, long ntodo, double scale, double zero,
 
 int ffi1fstr(unsigned char *input, long ntodo, double scale, double zero,
             char *cform, long twidth, char *output, int *status);
+int ffs1fstr(signed char *input, long ntodo, double scale, double zero,
+            char *cform, long twidth, char *output, int *status);
 int ffu2fstr(unsigned short *input, long ntodo, double scale, double zero,
             char *cform, long twidth, char *output, int *status);
 int ffi2fstr(short *input, long ntodo, double scale, double zero,
@@ -1034,7 +1104,7 @@ int compress2file_from_mem(
 #endif
 
 
-#if defined(vms) || defined(__vms) || defined(WIN32) || defined(__WIN32__) || defined(macintosh)
+#if defined(vms) || defined(__vms) || defined(WIN32) || defined(__WIN32__) || (defined(macintosh) && !defined(TARGET_API_MAC_CARBON))
 
 /* ================================================================== */
 /* A hack for nonunix machines, which lack strcasecmp and strncasecmp */
